﻿
<#
.SYNOPSIS
Starts a query using NET methods
.EXAMPLE
Show records

[System.Data.DataTable] $dt = Start-SQLQuery "SELECT name,create_date,user_access_desc,recovery_model_desc FROM sys.databases"

foreach($row in $dt.Rows)
{
   foreach($col in $dt.Columns)
   {
       Write-Host -NoNewline $row[$col]"`t" 
   }
   Write-Host ''
}

.EXAMPLE
 Show column captions

 [System.Data.DataTable] $dt = Start-SQLQuery "SELECT name,create_date,user_access_desc,recovery_model_desc FROM sys.databases"

 $columns=''
 foreach($col in $dt.Columns)
 {
    $columns += $col.Caption + "`t "
 }
 echo $columns

.EXAMPLE
Iterate through records

[System.Data.DataTable]$dt=Start-SQLQuery -instance TURING -database master -query "SELECT  name FROM sys.databases"  -UseSQLAuth -UserName sa -Password "****"
foreach($dr in $dt.Rows) { echo $dr["ci_name"] }

.EXAMPLE
Wuery a scalar
[System.Data.DataTable]$dt=Start-SQLQuery -instance TURING -database master -query "SELECT  count(*) AS cnt FROM sys.databases"
$dt.Rows[0]["cnt"]

#>
function Start-SQLQuery 
{
    param(
    [CmdletBinding()]
    #Query to run or ptocedure to start
    [Parameter(Mandatory=$true)]
    [string]$Query,
    #Database to use
    [string]$Database='master',
    #Instance
    [string]$instance='localhost',
    #Use Windows Authentication
    [switch]$UseSQLAuth=$false,
    #User name for non windows authentication connection
    [string]$UserName,
    #Password for non windows authentication connection
    [string]$Password,
    #Port number optional
    [int]$Port,
    #Timeout in seconds optional (set to 0 to wait infinity long)
    [int]$Timeout=0,
    #Treat as procedure
    [switch]$IsProcedure=$false
    )
    try
    {
        #Build connection string
        $connString = $Instance
        if($Port -gt 0)
        {
            $connString = "$connString,$Port"
        }
        if(! $UseSQLAuth)
        {
            $connString = "Server = $connString; Database = $Database; Integrated Security = True;Connection Timeout=$Timeout" 
        }
        else
        {
            $connString = "Server = $connString; Database = $database; User Id = $UserName; Password=$Password;Connection Timeout=$Timeout"
        }
            
        #connection
        [System.Data.SqlClient.SqlConnection]$SqlConnection = New-Object System.Data.SqlClient.SqlConnection
        $SqlConnection.ConnectionString = $connString

    	#query 
        if(! $IsProcedure)
        {
            [System.Data.SqlClient.SqlCommand]$SqlCmd = New-Object System.Data.SqlClient.SqlCommand
            $SqlCmd.CommandType = [System.Data.CommandType]::Text
    	    $SqlCmd.CommandText = $Query
        }
        else
        {
            [System.Data.SqlClient.SqlCommand]$SqlCmd = New-Object System.Data.SqlClient.SqlCommand
            $SqlCmd.CommandType = [System.Data.CommandType]::StoredProcedure
    	    $SqlCmd.CommandText = $Query
        }
    	$SqlCmd.Connection = $SqlConnection
		$sqlCmd.CommandTimeout = $Timeout

        #Data adapter
    	[System.Data.SqlClient.SqlDataAdapter]$SqlAdapter = New-Object System.Data.SqlClient.SqlDataAdapter
    	$SqlAdapter.SelectCommand = $SqlCmd
    	$DataSet = New-Object System.Data.DataSet

        #Start query| Out-Null required
    	$SqlAdapter.Fill($DataSet)  | Out-Null
    	$SqlConnection.Close()
    		 
        #Return data
        return ,$DataSet.Tables[0]
    }
    catch 
    {
        #In case of error return $null
        Write-Error "$($MyInvocation.MyCommand): Error during query execution, details: $($_.Exception.Message)" 
        return $null
    }
 } 
 

 del c:\temp\test_ps.bak
 del c:\temp\test_ps1.bak
 Start-SQLQuery -Query "Backup database test_ps to disk='c:\temp\test_ps.bak' with copy_only"
 copy C:\temp\test_ps.bak '\\turing\c$\temp\test_ps1.bak'
 Start-SQLQuery -instance Turing -Query "Restore database test_ps1 from disk='c:\temp\test_ps1.bak' with replace"


 Start-SQLQuery -Query 'sys.sp_helpdb' -instance turing -IsProcedure

 Start-SQLQuery -Query 'select * from sys.databases' -instance turing -IsProcedure
